//
//  XSPEC12  November 2003
//
//

#include <XSUser/Global/Global.h>
#include <XSUser/Global/XSGlobal.h>
#include <XSModel/GlobalContainer/ModelContainer.h>
#include <XSUtil/Parse/XSparse.h>
#include <XSUtil/FunctionUtils/FunctionUtility.h>
#include <XSFit/Fit/Fit.h>
#include <XSContainer.h>
#include <XSstreams.h>
#include <xsTypes.h>

int
XSGlobal::xsCosmo(ClientData cdata,Tcl_Interp* tclInterp,int objc, Tcl_Obj* CONST  objv[])
{
   using namespace XSContainer;
   if ( objc == 1)
   {
      tcout << "\n Cosmology parameters: H0 = " << models->cosmo().H0 
            << " q0 = " << models->cosmo().q0
            <<  " Lambda0 = " << models->cosmo().lambda0 << std::endl; 
   }
   else
   {
      string arg(Tcl_GetString(objv[1]));  
      if (arg.size() && arg[0] == '?')
      {
         XSGlobal::printDocs(static_cast<char*>(cdata),"?");      
      }  
      else
      {
         StringArray inArgs;
         StringArray params;
         IntegerArray iParams;
         for (int i=1; i<objc; ++i)
         {
            inArgs.push_back(Tcl_GetString(objv[i]));
         }
         XSparse::collectParams(inArgs, iParams, params);
         bool isChanged = false;
         for (size_t i=0; i<iParams.size(); ++i)
         {
            Real value=0.0;
            int iPar = iParams[i];
            if (iPar >= 0 && iPar < 3)
            {
               if (XSutility::isReal(params[i], value))
               {
                  switch (iPar)
                  {
                     case 0:
                        models->cosmo().H0 = value; 
                        FunctionUtility::setH0(value);
                        isChanged = true;
                        break;
                     case 1:
                        models->cosmo().q0 = value; 
                        FunctionUtility::setq0(value);
                        isChanged = true;
                        break;
                     case 2:
                        models->cosmo().lambda0 = value;
                        FunctionUtility::setlambda0(value); 
                        isChanged = true;
                        break;
                     default:
                        break;
                  }
               }
               else 
               {
                  string errPar;
                  switch (iPar)
                  {
                     case 0:
                        errPar = "H0";
                        break;
                     case 1:
                        errPar = "q0";
                        break;
                     case 2:
                        errPar = "lambda0";
                        break;
                     default:
                        break;
                  }
                  tcout << "\n***Invalid input ignored for cosmo parameter "
                        << errPar << ": " <<params[i]
                        << std::endl;
               } 
            } // end if iPar in valid range
         } // end params loop
         if (isChanged)
         {
            // Unfortunately we have no way of knowing at the moment if
            // a model contains a component that is affected by the cosmo
            // change.  Therefore we'll just have to recalculate all the
            // models.
            ModelMap::const_iterator itMod = models->modelSet().begin();
            ModelMap::const_iterator itEnd = models->modelSet().end();
            while (itMod != itEnd)
            {
               Model* mod = itMod->second;
               mod->setComputeFlag(true);
               mod->calculate();
               ++itMod;
            }
            fit->Update();
         }      
      } // end if not '?'   
   }
   return globalData->autoSave(TCL_OK);
}
